package model;

import java.util.ArrayList;
import java.util.HashSet;

/*
 * It is important to see how call by value works.
 */

public class RecursiveMethods {
	/*
	 * Return the number of ways for climbing a staircase
	 * 	with height of `h` steps, subject to the rule that
	 * 	each climb can take up to `n` steps.
	 */
	public HashSet<ArrayList<Integer>> climbStrategies(int h, int n) {
		HashSet<ArrayList<Integer>> solutionSet = new HashSet<>();
		// How does the call by value work to ensure only one instance of	
		// HashSet is being updated among recursive calls.
		climbStrategiesHelper(h, n, solutionSet, new ArrayList<Integer>());
		return solutionSet;
	}
	
	private void climbStrategiesHelper(int h, int n, 
			HashSet<ArrayList<Integer>> solutionSet, ArrayList<Integer> solutionSoFar) {
		if(h < 0) { // Gone beyond the top of the staircase.
			// Solution constructed so far is not valid, do not add it to the set.
		}
		else if(h == 0) {
			// Solution constructed so far is valid for reaching to the top.
			solutionSet.add(solutionSoFar);
		}
		else {
			// Make sure there's no interference of the solutions to be constructed
			// over different branches.
			for(int i = 1; i <= n; i ++) {
				ArrayList<Integer> currentSolution = new ArrayList<>(solutionSoFar);
				currentSolution.add(i);
				climbStrategiesHelper(h - i, n, solutionSet, currentSolution);
			}
			
		}
	}
}
 






